from .anabasii import Anabasii
from .basic_script import BasicScript
from .cooper import Cooper
from .tlacuilo import Tlacuilo


class Hersir(BasicScript):
    """Run all the steps to compile, upload and run a workflow to a running cluster"""

    def __get_scripts(self):
        return [
            Tlacuilo(self._parser),
            Anabasii(self._parser),
            Cooper(self._parser)
        ]

    def get_arguments(self):
        """
        Get the arguments to configure current script
        :return: list
        """
        arguments = []
        for script in self.__get_scripts():
            arguments.extend(script.get_arguments())

        return arguments

    def run(self, args, injector):
        """
        Run the steps in order to compile, upload and run a workflow
        :param args: Namespace
        :param injector: Injector
        """
        for script in self.__get_scripts():
            script.run(args, injector)



import os

from injector import inject


class ExcelWriter(object):
    """Generates the Excel files"""

    @inject(workbook='workbook', filesystem='filesystem', logger='logger')
    def __init__(self, workbook, filesystem, logger):
        """
        Initialize the class
        :param workbook: Workbook
        :param filesystem: Filesystem
        :param logger: Logging
        """
        super(ExcelWriter, self).__init__()
        self.__workbook = workbook
        self.__filesystem = filesystem
        self.__logger = logger

    def __init_count_sheet(self):

        count_sheet = self.__workbook.create_sheet('List & Count')

        return count_sheet

    def __init_tables_sheet(self):

        tables_sheet = self.__workbook.create_sheet('Tables information')
        return tables_sheet

    def __add_rows_sheet(self, table_name, columns, rows):

        rows_sheet = self.__workbook.create_sheet(table_name[:30])
        rows_sheet.append(['Table', table_name])
        rows_sheet.append(columns)

        for row in rows:
            try:
                rows_sheet.append(row)
            except:
                rows_sheet.append(['PARSE ROW ERROR'])

    def generate_excel_file(self, tables_information, db_name, location):
        """
        Generate all the excel file content given tables list and save it into a location
        :param tables_information: dict
        :param db_name: string
        :param location: string
        :return: boolean
        """
        count_sheet = self.__init_count_sheet()
        tables_sheet = self.__init_tables_sheet()

        self.__logger.debug('Adding sheet with the table list and count for each table')
        count_sheet.append(['Database schema', 'Table name', '#rows'])
        for table_name in tables_information:
            count_sheet.append([db_name, table_name, tables_information[table_name]['count']])

            self.__logger.debug('Adding {table_name} table information'.format(table_name=table_name))
            tables_sheet.append([table_name])
            tables_sheet.append(['FIELD', 'TYPE', 'MAX_LENGTH', 'NULLABLE', 'DEFAULT VALUE'])

            columns = []
            for column in tables_information[table_name]['columns']:
                tables_sheet.append([column['column_name'], column['data_type'], column['character_maximum_length'],
                                     column['is_nullable'], column['column_default']])
                columns.append(column['column_name'])

            tables_sheet.append([])
            tables_sheet.append([])

            if 'rows' in tables_information[table_name] and tables_information[table_name]['rows']:
                self.__logger.debug('Adding rows information to {table_name} sheet'.format(table_name=table_name))
                self.__add_rows_sheet(table_name, columns, tables_information[table_name]['rows'])

        self.__filesystem.mkdir(location)
        self.__logger.debug('Saving excel file {db_name} into {location}'.format(db_name=db_name, location=location))
        return self.__workbook.save(os.path.join(location, db_name + '.xlsx'))



import os.path


class DeployConfiguration(object):
    """Manage the deploy configuration file"""

    def __init__(self, environment, configuration_parser):
        """
        Initialize the class
        :param environment: string
        :param configuration_parser: ConfigParser
        """
        super(DeployConfiguration, self).__init__()
        self.__environment = environment
        self.__configuration_file_path = os.path.join(os.path.expanduser('~'), '.slippinj')
        self.__configuration_parser = configuration_parser

    def file_exists(self):
        """
        Check if configuration file is present in filesystem
        :return: boolean
        """
        return os.path.exists(self.__configuration_file_path)

    def environment_exists(self):
        """
        Check if environment configuration exists in configuration file
        :return: boolean
        """
        if self.file_exists():
            self.__configuration_parser.read(self.__configuration_file_path)

            return self.__configuration_parser.has_section(self.__environment)

        return False

    def get(self, key):
        """
        Get the given parameter from the configuration file for the current environment
        :param key: string
        :return: string
        """
        if self.file_exists() and self.environment_exists():
            self.__configuration_parser.read(self.__configuration_file_path)

            return self.__configuration_parser.get(self.__environment, key) if self.__configuration_parser.has_option(
                self.__environment, key) else False

        return False

    def set(self, key, value):
        """
        Save given key and value into given configuration file inside current environment
        :param key: string
        :param value: string
        """
        self.__configuration_parser.read(self.__configuration_file_path)

        if not self.environment_exists():
            self.__configuration_parser.add_section(self.__environment)

        self.__configuration_parser.set(self.__environment, key, value)

        self.__configuration_parser.write(open(self.__configuration_file_path, 'w'))



from injector import inject, AssistedBuilder
from slippinj.cli.deploy_configuration import DeployConfiguration


class DefaultConfiguration(object):
    """Get the default configuration for the workflows"""

    @inject(deploy_configuration=AssistedBuilder(DeployConfiguration), configuration_parser='configuration_parser')
    def __init__(self, deploy_configuration, configuration_parser):
        """
        Initialize the class
        :param deploy_configuration: DeployConfiguration
        :param configuration_parser: ConfigParser
        """
        super(DefaultConfiguration, self).__init__()

        self.__deploy_configuration = deploy_configuration
        self.__configuration_parser = configuration_parser

    def get(self, environment, arguments, workflow_configuration):
        """
        Get configuration parameters that are common to the workflows
        :param environment: string
        :param arguments: Namespace
        :param workflow_configuration: dict
        :return: dict
        """
        default_variables = ['hive_metastore_bucket', 'hdfs_deploy_folder']
        default_configuration = {}
        interactive_provided = False
        deploy_configuration = self.__deploy_configuration.build(environment=environment,
                                                                 configuration_parser=self.__configuration_parser)
        args = vars(arguments)

        for variable in default_variables:
            if variable in args and False != args[variable]:
                default_configuration[variable] = args[variable]
                interactive_provided = True
            elif variable in workflow_configuration:
                default_configuration[variable] = workflow_configuration[variable]
            elif deploy_configuration.get(variable):
                default_configuration[variable] = deploy_configuration.get(variable)
            else:
                default_configuration[variable] = raw_input(
                    'Please, provide the {var_name} value: '.format(var_name=variable.replace('-', ' ')))
                interactive_provided = True

        if interactive_provided and 'y' == (
                raw_input('Would you like to save the provided information in the config file: [Y/N] ')).lower():
            for key in default_configuration:
                deploy_configuration.set(key, default_configuration[key])

        return default_configuration



from botocore.exceptions import WaiterError
from injector import inject


class RunJobFlow(object):
    """Run a job flow based on a given configuration"""

    @inject(aws_emr_client='aws_emr_client', logger='logger', job_flow_configuration='job_flow_configuration')
    def __init__(self, aws_emr_client, logger, job_flow_configuration):
        """
        Initialize the class
        :param aws_emr_client: EMR.client
        :param logger: Logging
        """
        super(RunJobFlow, self).__init__()
        self.__aws_emr_client = aws_emr_client
        self.__logger = logger
        self.__job_flow_configuration = job_flow_configuration

    def run_cluster(self, cluster_configuration):
        """
        Launch a cluster based on given configuration
        :param cluster_configuration: string
        """
        aws_arguments = self.__job_flow_configuration.convert_to_arguments(cluster_configuration)

        self.__logger.debug('Launching cluster with given configuration')
        response = self.__aws_emr_client.run_job_flow(**aws_arguments)
        self.__logger.info('Cluster launched with ID: {cluster_id}'.format(cluster_id=response['JobFlowId']))
        try:
            waiter = self.__aws_emr_client.get_waiter('cluster_running')
            self.__logger.info('Waiting for cluster to be ready')
            waiter.wait(ClusterId=response['JobFlowId'])
            self.__logger.info('Cluster up and running ID: {cluster_id}'.format(cluster_id=response['JobFlowId']))
        except WaiterError:
            self.__logger.error('Cluster creation failed')


from string import Template

from ..generator.wf_templates import WfTemplatesRender


class HiveTemplatesRender(WfTemplatesRender):
    """Process templates to generate custom Hive scripts"""

    def _generate_output_filename(self, filename, template_vars):
        return Template(filename).substitute(template_vars).replace('.j2', '')

    def render_hive_folder(self, template_dir, template, output_folder, tables_configuration):
        """
        Render all the templates related to hive
        :param template_dir: string
        :param template: string
        :param output_folder: string
        :param tables_configuration: dict
        """
        for table_name in tables_configuration:
            tables_configuration[table_name]['table_name'] = table_name
            self.render_workflow_folder(template_dir, template, output_folder, tables_configuration[table_name])



from injector import inject


class HDFSFilesystem(object):
    """Run HDFS commands inside the cluster"""

    @inject(emr_cluster='emr_cluster', logger='logger')
    def __init__(self, emr_cluster, logger):
        """
        Initialize the class
        :param emr_cluster: EMRCluster
        """
        super(HDFSFilesystem, self).__init__()
        self.__emr_cluster = emr_cluster
        self.__logger = logger

    def rmdir(self, path, cluster_id):
        self.__logger.debug('Removing {path} from cluster {cluster}'.format(path=path, cluster=cluster_id))
        return self.__emr_cluster.exec_command('hdfs dfs -rm -r -f ' + path, cluster_id)

    def mkdir(self, path, cluster_id):
        self.__logger.debug('Creating directory {path} in cluster {cluster}'.format(path=path, cluster=cluster_id))
        return self.__emr_cluster.exec_command('hdfs dfs -mkdir -p ' + path, cluster_id)

    def put(self, src, dest, cluster_id):
        self.__logger.debug(
            'Uploading {src} to {dest} in cluster {cluster}'.format(src=src, dest=dest, cluster=cluster_id))
        return self.__emr_cluster.exec_command('hdfs dfs -put ' + src + ' ' + dest, cluster_id)



class WfConfigurationObject(dict):
    """Stores the workflow configuration"""

    def __init__(self, configuration={}):
        dict.__init__(self, configuration)

    def __getattr__(self, item):
        try:
            return self.__getitem__(item)
        except KeyError:
            raise AttributeError(item)

    def __setattr__(self, item, value):
        if self.__dict__.has_key(item):
            dict.__setattr__(self, item, value)
        else:
            self.__setitem__(item, value)



from injector import inject


class WorkflowConfiguration(object):
    """Read the configuration for the given workflow"""

    @inject(yaml_configuration='yaml_configuration')
    def __init__(self, yaml_configuration):
        """
        Initialize the class
        :param yaml_configuration: YamlConfiguration
        """
        super(WorkflowConfiguration, self).__init__()

        self.__yaml_configuration = yaml_configuration

    def get_workflow_configuration(self, configuration_file):
        """
        Get the workflow configuration given the file or files where it can be found
        :param configuration_file: string or list, if a list is given all the files will be read sorted as they are provided
        :return: dict
        """
        return self.__yaml_configuration.read_config_file(configuration_file) if type(
            configuration_file) == str else self.__yaml_configuration.read_multiple_config_files(configuration_file)



import pymssql
import re
import unicodedata

from injector import inject, AssistedBuilder


class Sqlserver(object):
    """Wrapper to connect to SQL Servers and get all the metastore information"""

    @inject(mssql=AssistedBuilder(callable=pymssql.connect), logger='logger')
    def __init__(self, mssql, logger, db_host=None, db_user='root', db_name=None, db_schema=None, db_pwd=None, db_port=None):
        """
        Initialize the SQLServer driver to get all the tables information
        :param mssql: Pymssql
        :param logger: Logger
        :param db_host: string
        :param db_user: string
        :param db_name: string
        :param db_schema: string
        :param db_pwd: string
        :param db_port: int
        """
        super(Sqlserver, self).__init__()

        self.__db_name = db_name
        self.__db_schema = db_schema if None != db_schema else 'dbo'
        self.__conn = mssql.build(server=db_host, user=db_user, password=db_pwd, database=db_name,
                                  port=db_port if None != db_port else 1433)

        self.__column_types = {
            'uniqueidentifier': 'string',
            'datetime': 'timestamp',
            'nvarchar': 'string',
            'money': 'double',
            'decimal': 'double',
            'bit': 'boolean',
            'float': 'double',
            'varbinary': 'binary',
            'char': 'string',
            'date': 'timestamp',
            'datetime': 'timestamp',
            'datetime2': 'timestamp',
            'datetimeoffset(2)': 'timestamp',
            'image': 'string',
            'integer': 'int',
            'nchar': 'string',
            'ntext': 'string',
            'numeric': 'double',
            'nvarchar(max)': 'double',
            'real': 'double',
            'smalldatetime': 'timestamp',
            'smallmoney': 'double',
            'text': 'string',
            'time': 'timestamp',
            'udt': 'binary',
            'varbinary(max)': 'string',
            'varchar': 'string',
            'varchar(max)': 'string',
            'xml': 'string'
        }

        self.__illegal_characters = re.compile(r'[\000-\010]|[\013-\014]|[\016-\037]')

        self.__logger = logger

    def __join_tables_list(self, tables):
        return ','.join('\'%s\'' % table for table in tables)

    def __get_table_list(self, table_list_query=False):

        self.__logger.debug('Getting table list')
        query = 'SELECT table_name FROM information_schema.tables WHERE table_catalog = %(db_name)s and table_schema = %(schema)s {table_list_query}'.format(
            table_list_query=' AND ' + table_list_query if table_list_query else '')
        cursor = self.__conn.cursor(as_dict=True)
        cursor.execute(query, {'db_name': self.__db_name, 'schema': self.__db_schema})

        self.__logger.debug('Found {count} tables'.format(count=cursor.rowcount))

        return map(lambda x: x['table_name'], cursor.fetchall())

    def __get_tables_to_exclude(self, tables):
        return self.__get_table_list('table_name NOT IN ({tables})'.format(tables=self.__join_tables_list(tables)))

    def __get_columns_for_tables(self, tables):

        self.__logger.debug('Getting columns information')
        info_query = 'SELECT table_name, column_name, data_type, character_maximum_length, is_nullable, column_default FROM information_schema.columns WHERE table_name IN ({tables}) AND table_catalog=%(db_name)s AND table_schema=%(schema)s'.format(
            tables=self.__join_tables_list(tables))

        cursor = self.__conn.cursor(as_dict=True)
        cursor.execute(info_query, {'db_name': self.__db_name, 'schema': self.__db_schema})

        tables_information = {}
        for row in cursor.fetchall():
            self.__logger.debug(
                'Columns found for table {table}'.format(table=row['table_name']))
            if not row['table_name'] in tables_information:
                tables_information[row['table_name']] = {'columns': []}

            tables_information[row['table_name']]['columns'].append({
                'column_name': row['column_name'],
                'data_type': row['data_type'] if row['data_type'] not in self.__column_types else self.__column_types[
                    row['data_type']],
                'character_maximum_length': row['character_maximum_length'],
                'is_nullable': row['is_nullable'],
                'column_default': row['column_default'],
            })

        return tables_information

    def __get_count_for_tables(self, tables):

        tables_information = {}
        cursor = self.__conn.cursor()
        for table in tables:
            try:
                self.__logger.debug('Getting count for table {table}'.format(table=table))
                info_query = 'SELECT COUNT(*) FROM {schema}.{table}'.format(table=table, schema=self.__db_schema)
                cursor.execute(info_query)
                tables_information[table] = {'count': cursor.fetchone()[0]}
            except:
                pass

        return tables_information

    def __get_top_for_tables(self, tables, top=30):

        tables_information = {}

        cursor = self.__conn.cursor()
        for table in tables:
            tables_information[table] = {'rows': []}
            if top > 0:
                try:
                    self.__logger.debug('Getting {top} rows for table {table}'.format(top=top, table=table))
                    cursor.execute('SELECT TOP {top} * FROM {table}'.format(top=top, table=table))
                    for row in cursor.fetchall():
                        table_row = []
                        for column in row:
                            try:
                                if type(column) is unicode:
                                    column = unicodedata.normalize('NFKD', column).encode('iso-8859-1', 'ignore')
                                else:
                                    column = str(column).decode('utf8').encode('iso-8859-1')
                                    if self.__illegal_characters.search(column):
                                        column = 'Hexadecimal'
                                if column == 'None':
                                    column = 'NULL'
                            except:
                                column = 'Parse_error'

                            table_row.append(column)

                        tables_information[table]['rows'].append(table_row)

                except pymssql.ProgrammingError:
                    tables_information[table]['rows'].append(
                        'Error getting table data {error}'.format(error=pymssql.ProgrammingError.message))

        return tables_information

    def get_all_tables_info(self, table_list, table_list_query, top_max):
        """
        Return all the tables information reading from the Information Schema database
        :param table_list: string
        :param table_list_query: string
        :param top_max: integer
        :return: dict
        """
        tables_to_exclude = {}

        if table_list:
            tables = map(lambda x: unicode(x), table_list.split(','))
            tables_to_exclude = self.__get_tables_to_exclude(tables)
        else:
            tables = self.__get_table_list(table_list_query)

        tables_counts = self.__get_count_for_tables(tables)
        tables_columns = self.__get_columns_for_tables(tables)
        tables_top = self.__get_top_for_tables(tables, top_max)

        tables_info = {'tables': {}}
        for table in tables_counts:
            tables_info['tables'][table] = {}
            tables_info['tables'][table].update(tables_columns[table])
            tables_info['tables'][table].update(tables_counts[table])
            tables_info['tables'][table].update(tables_top[table])

        if tables_to_exclude:
            tables_info['excluded_tables'] = tables_to_exclude

        return tables_info



import os

from injector import inject


class WfTemplatesRender(object):
    """Proccess the templates to generate a whole Oozie Workflow using Jinja2 syntax"""

    @inject(templates_lib='templates_lib', filesystem='filesystem', logger='logger')
    def __init__(self, templates_lib, filesystem, logger):
        """
        Initialize the class
        :param templates_lib: TemplatesLib
        :param filesystem: Filesystem
        :param logger: Logging
        """
        super(WfTemplatesRender, self).__init__()

        self.__templates_lib = templates_lib
        self.__filesystem = filesystem
        self.__logger = logger

    def _generate_output_filename(self, filename, template_vars):
        self.__logger.debug('Output from WFTemplatesRender')
        return filename.replace('.j2', '')

    def __render_template_files(self, base_directory, output_folder, template_dir, template_vars, root_folder, files):
        files_output_folder = os.path.join(output_folder, os.path.basename(root_folder).replace(template_dir, ''))
        for template_file in files:
            self.__logger.debug('Rendering template file {template_file}'.format(template_file=template_file))
            full_template_file = os.path.join(root_folder.replace(base_directory, ''), template_file)

            output_file = os.path.join(files_output_folder, self._generate_output_filename(template_file, template_vars))
            self.__logger.debug('Writing rendered template into {output_file}'.format(output_file=output_file))
            self.__filesystem.write_file(
                output_file,
                self.__templates_lib.render(base_directory, full_template_file, template_vars)
            )

    def __get_template_files(self, base_directory, template_dir):
        self.__logger.debug('Getting templates from directory {base_directory}'.format(base_directory=base_directory))
        template_files = []

        for root, dirs, files in os.walk(os.path.join(base_directory, template_dir)):
            template_files.append({'files': files, 'root': root})

        return template_files

    def render_workflow_folder(self, base_directory, template_dir, output_folder, template_vars):
        """
        From the given template render all the files found inside and recreate the structure into output directory
        :param base_directory: string
        :param template_dir: string
        :param output_folder: string
        :param template_vars: dict
        """
        for template_files in self.__get_template_files(base_directory, template_dir):
            self.__render_template_files(
                base_directory,
                output_folder,
                template_dir,
                template_vars,
                template_files['root'],
                template_files['files']
            )



class GroupTablesConfiguration(object):
    def _is_mapping_needed(self, column_type):
        return column_type != 'string' and column_type != 'int'

    def _get_column_definition(self, column):
        return {'name': column['column_name'], 'type': column['data_type']}



from .group_tables_configuration import GroupTablesConfiguration


class IncrementalTablesConfiguration(GroupTablesConfiguration):
    """Generate the incremental tables configuration"""

    def __get_default_configuration(self):
        return {
            'sqoop_options': {
                'split-by': '',
                'where': '',
                'map-column-hive': '',
            },
            'column_definition': {
                'partition_column': 'dt',
                'column_split': '',
                'columns': []
            }
        }

    def get_table_configuration(self, columns):
        """
        Return configuration for given table, considering it is an incremental table
        :param columns: list
        :return: dict
        """
        table_configuration = self.__get_default_configuration()

        table_columns = []
        map_column_hive_fields_list = []
        for column in columns:
            if column['data_type'] == 'timestamp' and not table_configuration['sqoop_options']['split-by']:
                table_configuration['sqoop_options']['split-by'] = column['column_name']
                table_configuration['column_definition']['column_split'] = column['column_name']
                table_configuration['sqoop_options']['where'] = column['column_name'] + ' = \'${initDate}\''

            if self._is_mapping_needed(column['data_type']):
                map_column_hive_fields_list.append(column['column_name'] + '=' + column['data_type'])

            table_columns.append(self._get_column_definition(column))

        table_configuration['column_definition']['columns'] = table_columns
        table_configuration['sqoop_options']['map-column-hive'] = ','.join(map_column_hive_fields_list)

        if not table_configuration['sqoop_options']['split-by']:
            raise ValueError(
                'Given table can\'t be imported as incremental because not timestamp column has been found')

        return table_configuration



import os

import sys

from .basic_script import BasicScript


class Anabasii(BasicScript):
    """Upload the compiled code to the cluster"""

    def get_arguments(self):
        """
        Get the arguments to configure current script
        :return: list
        """
        return [
            {
                'short': '-c',
                'long': '--cluster-id',
                'help': 'Provide a cluster id to work with, in case you don\'t provide one a list of available clusters will be shown to select one',
                'default': False
            },
            {
                'short': '-m',
                'long': '--hive-metastore-bucket',
                'help': 'Bucket where the Hive tables are stored',
                'default': False
            },
            {
                'short': '-d',
                'long': '--hdfs-deploy-folder',
                'help': 'Folder on HDFS to deploy the code',
                'default': False
            },
            {
                'short': '-w',
                'long': '--wf-dir',
                'help': 'Folder where all the workflows files are present',
                'required': True
            }
        ]

    def run(self, args, injector):
        """
        Run the component to upload the code to the cluster
        :param args: Namespace
        :param injector: Injector
        """
        logger = injector.get('logger')
        cluster_id = args.cluster_id if False != args.cluster_id else injector.get('interactive_cluster_id').get()

        if args.script not in __name__:
            logger.info('Getting workflow configuration')
            configuration = self.get_wf_configuration(args, injector)
            configuration.cluster_id = cluster_id

            wf_compiled_dir = configuration.output_directory if configuration.output_directory else args.wf_dir
        else:
            wf_compiled_dir = args.wf_dir
            configuration = injector.get('interactive_default_configuration').get('devel', args, {})

        logger.info('Uploading {wf_dir} to the cluster {cluster_id}'.format(wf_dir=wf_compiled_dir, cluster_id=cluster_id))
        injector.get('emr_deploy').upload_code(wf_compiled_dir, cluster_id, configuration['hdfs_deploy_folder'],
                                               args.wf_dir.strip(os.sep).split(os.sep)[-1])



import glob
import os
import shutil
import tarfile
import tempfile
import time

from injector import inject


class Filesystem(object):
    """Manage the filesystem"""

    @inject(logger='logger')
    def __init__(self, logger):
        """
        Initialize the class
        :param logger: Logging
        """
        super(Filesystem, self).__init__()
        self.__tmp_dir = None
        self.__logger = logger

    def mkdir(self, output_dir):
        """
        Create a directory into filesystem
        :param output_dir: string
        :return: boolean
        """
        output_dir = os.path.dirname(output_dir)
        self.__logger.debug('Generating directory {output_dir}'.format(output_dir=output_dir))
        return os.mkdir(output_dir) if False == os.path.exists(output_dir) else True

    def write_file(self, output_file, content):
        """
        Write given content into file
        :param output_file: string
        :param content: string
        """
        self.mkdir(output_file)

        self.__logger.debug('Writing content into {output_file}'.format(output_file=output_file))
        f = open(output_file, 'w')
        f.write(content)
        f.close()

    def generate_temp_dir(self):
        """
        Generate a temporary directory and return the path to it
        :return: string
        """
        if not self.__tmp_dir:
            self.__tmp_dir = tempfile.mkdtemp()
        self.__logger.debug('Generating temporary directory')
        return self.__tmp_dir

    def remove_temp_dir(self):
        """
        Remove the temporary directory created using self.generate_temp_dir
        """
        if self.__tmp_dir:
            self.__logger.debug('Removing previously generated temporary directory')
            shutil.rmtree(self.__tmp_dir)

    def find_properties_file(self, wf_folder):
        """
        Find all the properties file inside the given folder
        :param wf_folder: string
        :return: list
        """
        self.__logger.debug('Scanning all properties files inside {folder}'.format(folder=wf_folder))
        return (
            glob.glob(os.path.join(wf_folder, '*.properties')) + glob.glob(
                os.path.join(wf_folder, '**', '*.properties')))

    def generate_tar_file(self, wf_folder, workflow_name=None):
        """
        Generate a tar file and compress inside all the workflow code, and returns the path to the newly created file
        :param wf_folder: string
        :param workflow_name: string
        :return: string
        """
        self.__logger.debug('Compressing {wf_folder} content inside tar file'.format(wf_folder=wf_folder))
        tar_filename = workflow_name + str(time.time()) + '.tar.gz'
        tar_name = os.path.join(wf_folder, tar_filename)
        tar = tarfile.open(name=tar_name, mode='w:gz')
        tar.add(wf_folder, arcname=workflow_name)
        tar.close()

        return os.path.join(wf_folder, tar_filename)



from .basic_script import BasicScript


class Valet(BasicScript):
    """Provision the cluster with the needed software"""

    def get_arguments(self):
        """
        Get the arguments to configure current script
        :return: list
        """
        return [
            {
                'short': '-p',
                'long': '--playbook',
                'help': 'Ansible playbook to execute in order to provision the cluster'
            },
            {
                'short': '-c',
                'long': '--cluster-id',
                'help': 'Cluster to execute the provisioning'
            },
            {
                'short': '-x',
                'long': '--create-cluster',
                'help': 'Configuration to launch a cluster on the given AWS profile'
            }
        ]

    def run(self, args, injector):
        """
        Run the component to provision the cluster using Ansible
        :param args: Namespace
        :param injector: Injector
        """
        if args.playbook:
            cluster_id = args.cluster_id if None != args.cluster_id else injector.get('interactive_cluster_id').get()

            injector.get('logger').info('Running provided playbook on the cluster')
            injector.get('ansible_client').run_playbook(args.playbook, cluster_id)
        elif args.create_cluster:
            injector.get('logger').info(
                'Launching cluster based on {cluster_config} configuration'.format(cluster_config=args.create_cluster))
            injector.get('job_flow').run_cluster(
                injector.get('yaml_configuration').read_config_file(args.create_cluster))
        else:
            raise RuntimeError('Not arguments where provided to run Valet')



from injector import inject, AssistedBuilder
from jinja2 import Environment, FileSystemLoader


class TemplatesLib(object):
    """Manage all the templates library interaction"""

    @inject(loader=AssistedBuilder(FileSystemLoader), environment=AssistedBuilder(Environment))
    def __init__(self, loader, environment):
        """
        Initialize the class
        :param loader: FilesystemLoader
        :param environment: Environment
        """
        super(TemplatesLib, self).__init__()

        self.__loader = loader
        self.__environment = environment

    def __get_lib(self, base_directory):
        return self.__environment.build(
            loader=self.__loader.build(searchpath=base_directory),
            trim_blocks=True,
            extensions=['jinja2.ext.do']
        )

    def render(self, base_directory, template_file, template_vars):
        """
        Render the given template with the template_vars
        :param base_directory: string Path where the template can be found
        :param template_file: string
        :param template_vars: dict
        :return: string
        """
        return self.__get_lib(base_directory).get_template(template_file).render(template_vars)



import arguments


class JobFlowConfigurationParser(object):
    """Given a configuration dict validate it and prepares to run a job flow"""

    def __init__(self):
        """
        Initialize the class
        """
        super(JobFlowConfigurationParser, self).__init__()
        self.__mandatory_parameters = [
            'name',
            'release_label',
            'instances.ec2_key_name',
            'availability_zone',
            'instances.master',
            'instances.core',
            'tags'
        ]

    def validate(self, configuration):
        """
        Validate if the given configuration has all the mandatory parameters configured
        :param configuration: dict
        :return: boolean
        """
        for key in self.__mandatory_parameters:
            if '.' in key:
                configuration_group = configuration
                for key_part in key.split('.'):
                    if key_part not in configuration_group:
                        raise AttributeError('{key}  not found in configuration file'.format(key=key))
                    configuration_group = configuration_group[key_part]
            else:
                if key not in configuration:
                    raise AttributeError('{key}  not found in configuration file'.format(key=key))

        return True

    def __to_camel_case(self, key):
        converted = ''
        for i in key.split('_'):
            converted += i.capitalize()

        return converted

    def convert_to_arguments(self, configuration):
        """
        Convert from the configuration dict to a valid dict to send as arguments to the AWS API
        :param configuration: dict
        :return: dict
        """
        configuration_converted = {}

        for key in configuration:
            new_key = self.__to_camel_case(key)
            if arguments.parser_exists(key):
                configuration_converted.update(arguments.get_parser(key).parse(configuration[key]))
            else:
                configuration_converted[new_key] = configuration[key]

        return configuration_converted



class ConfigurationsArgument(object):
    """Parse the configurations argument"""

    def __init__(self):
        """
        Initialize the class
        """
        super(ConfigurationsArgument, self).__init__()
        self.__keys_to_change = [
            'configurations',
            'classification',
            'properties'
        ]

    def __key_to_capital(self, configuration_dict):
        if isinstance(configuration_dict, list):
            return [self.__key_to_capital(v) for v in configuration_dict]
        elif isinstance(configuration_dict, dict):
            return dict(
                (k.capitalize() if k.lower() in self.__keys_to_change else k, self.__key_to_capital(v)) for k, v in
                configuration_dict.iteritems())
        else:
            return configuration_dict

    def parse(self, configuration):
        """
        Parse configurations from the configuration file and return it formatted
        :param configuration: dict
        :return: dict
        """
        return {
            'Configurations': list(self.__key_to_capital(configuration))
        }



import os
from collections import namedtuple

from ansible.executor.playbook_executor import PlaybookExecutor
from ansible.inventory import Inventory
from ansible.parsing.dataloader import DataLoader
from ansible.vars import VariableManager
from injector import inject


class AnsibleClient(object):
    """Manage all the ansible execution requests"""

    @inject(emr_cluster='emr_cluster')
    def __init__(self, emr_cluster):
        """
        Initialize the class
        :param emr_cluster: EmrCluster
        """
        super(AnsibleClient, self).__init__()
        self.__emr_cluster = emr_cluster

    def run_playbook(self, playbook_path, cluster_id):
        """
        Run the given playbook using Ansible via API
        :param playbook_path: string
        :param cluster_id: string
        :return: string
        """
        if not os.path.exists(playbook_path):
            raise LookupError('Playbook to execute not found')

        loader = DataLoader()
        cluster_information = self.__emr_cluster.get_cluster_information(cluster_id)

        inventory = Inventory(loader=loader, variable_manager=VariableManager(),
                              host_list=',' + cluster_information['public_ip'])

        options = namedtuple('Options',
                             ['listtags', 'listtasks', 'listhosts', 'syntax', 'connection', 'module_path', 'forks',
                              'remote_user', 'private_key_file', 'ssh_common_args', 'ssh_extra_args', 'sftp_extra_args',
                              'scp_extra_args', 'become', 'become_method', 'become_user', 'verbosity', 'check'])

        options = options(listtags=False, listtasks=False, listhosts=False, syntax=False, connection='ssh',
                          module_path=None, forks=100, remote_user='hadoop',
                          private_key_file=self.__emr_cluster.get_pem_path(cluster_id), ssh_common_args=None,
                          ssh_extra_args=None, sftp_extra_args=None, scp_extra_args=None, become=True,
                          become_method='sudo', become_user='hadoop', verbosity=10, check=False)

        return PlaybookExecutor(playbooks=[playbook_path], inventory=inventory, variable_manager=VariableManager(),
                                loader=loader, options=options, passwords={}).run()



import os
from .basic_script import BasicScript


class Scribe(BasicScript):
    """Generate the documentation and basic configuration to run next Jimmy's steps based on database structure"""

    def get_arguments(self):
        """
        Get the arguments to configure current script
        :return: list
        """
        return [
            {
                'short': '-D',
                'long': '--db-driver',
                'help': 'Driver to use to connect to the database',
                'required': True
            },
            {
                'short': '-H',
                'long': '--db-host',
                'help': 'Database\'s hostname',
                'required': True
            },
            {
                'short': '-P',
                'long': '--db-port',
                'help': 'Database\'s port'
            },
            {
                'short': '-u',
                'long': '--db-user',
                'help': 'Database\'s username',
                'required': True
            },
            {
                'short': '-p',
                'long': '--db-pwd',
                'help': 'Database\'s password'
            },
            {
                'short': '-n',
                'long': '--db-name',
                'help': 'Database\'s name to get the information from'
            },
            {
                'short': '-s',
                'long': '--db-schema',
                'help': 'Database\'s schema to get the information from'
            },
            {
                'short': '-l',
                'long': '--table-list',
                'help': 'List of tables, separated by commas, to get information from'
            },
            {
                'short': '-E',
                'long': '--excel-only',
                'action': 'store_true',
                'help': 'If set, generate excel only for documentation purposes'
            },
            {
                'short': '-r',
                'long': '--max-records',
                'default': 30,
                'help': 'Set the max number of sample records per table, default 30.'
            },
            {
                'short': '-o',
                'long': '--output',
                'help': 'Directory where the output would be generated'
            },
            {
                'short': '-N',
                'long': '--yaml-db-name',
                'help': 'Database\'s name to be used in YML files'
            },
            {
                'short': '-w',
                'long': '--where',
                'help': 'Where clause to extend default query'
            },
            {
                'short': '-o',
                'long': '--output',
                'help': 'Directory where write all the files',
                'default': os.getcwd()
            }
        ]

    def run(self, args, injector):
        """
        Run the component to generate documentation and basic configuration files based on source database
        :param args: Namespace
        :param injector: Injector
        """
        logger = injector.get('logger')

        logger.info('Getting driver instance and connecting to database')
        db = injector.get('db_factory').get_driver(injector, args.db_driver, args.db_host, args.db_user, args.db_pwd,
                                                   db_port=args.db_port, db_name=args.db_name, db_schema=args.db_schema)

        logger.info('Getting tables information')
        tables_information = db.get_all_tables_info(args.table_list, args.where, args.max_records)

        logger.info('Writing tables information into excel file')
        injector.get('excel_writer').generate_excel_file(tables_information['tables'], args.db_name, args.output)

        if not args.excel_only:
            logger.info('Writing tables information into config files')
            injector.get('yaml_configuration_writer').generate_yaml_files(injector, tables_information, args.db_driver,
                                                                          args.db_host, args.db_port,
                                                                          args.db_user, args.db_name, args.db_pwd,
                                                                          args.output)



class TagsArgument(object):
    """Parse the tags argument"""

    def __init__(self):
        """
        Initialize the class
        """
        super(TagsArgument, self).__init__()

    def parse(self, configuration):
        """
        Parse tags from the configuration file and return it formatted
        :param configuration: dict
        :return: dict
        """
        return {
            'Tags': [{'Key': tag, 'Value': configuration[tag]} for tag in configuration]
        }



class TablesConfiguration(object):
    """Generate the tables configuration"""

    def __init__(self):
        """
        Initialize the class
        """
        self.INCREMENTAL_GROUP = 'incremental'
        self.OTHERS_GROUP = 'others'

    def __get_table_group(self, columns):
        for column in columns:
            if 'timestamp' == column['data_type']:
                return self.INCREMENTAL_GROUP

        return self.OTHERS_GROUP

    def generate_configuration(self, tables_information, injector):
        """
        Generate the basic configuration given all the tables information
        :param tables_information: dict
        :param injector: Injector
        :return: dict
        """
        tables_data = {}

        logger = injector.get('logger')
        if 'excluded_tables' in tables_information and tables_information['excluded_tables']:
            logger.debug('Configuring excluded tables')
            tables_data['excluded_tables'] = tables_information['excluded_tables']

        for table in tables_information['tables']:
            logger.debug('Generating configuration for table {table}'.format(table=table))
            table_group = self.__get_table_group(tables_information['tables'][table]['columns'])
            if not table_group in tables_data:
                tables_data[table_group] = {}

            tables_data[table_group][table] = injector.get(
                table_group + '_table_configuration'
            ).get_table_configuration(tables_information['tables'][table]['columns'])

        return tables_data



import os

from injector import inject


class PropertiesFile(object):
    """Get the properties file to run on the cluster"""

    @inject(filesystem='filesystem')
    def __init__(self, filesystem):
        """
        Initialize the class
        :param filesystem: Filesystem
        """
        super(PropertiesFile, self).__init__()

        self.__filesystem = filesystem

    def get(self, wf_dir):
        """
        Find all the properties file inside given workflow directory
        :param wf_dir: string
        :return: string
        """
        property_files = self.__filesystem.find_properties_file(wf_dir)

        print 'Properties files found:'
        print '[0] None'
        for index, property_file in enumerate(property_files):
            print '[{index}] {file}'.format(index=(index + 1), file=os.path.basename(property_file))

        file_to_run = int(raw_input('Select which file you would like to run: '))
        if file_to_run > 0:
            return property_files[(file_to_run - 1)]

        return False



from .group_tables_configuration import GroupTablesConfiguration


class OthersTableConfiguration(GroupTablesConfiguration):
    """Generate the others tables configuration"""

    def __get_default_configuration(self):
        return {
            'sqoop_options': {
                'm': '1',
                'map-column-hive': ''
            },
            'column_definition': {
                'columns': []
            }
        }

    def get_table_configuration(self, columns):
        """
        Return configuration for given table, considering it is a table classified into others group
        :param columns: list
        :return: dict
        """
        table_configuration = self.__get_default_configuration()

        table_columns = []
        map_column_hive_fields_list = []
        for column in columns:
            if self._is_mapping_needed(column['data_type']):
                map_column_hive_fields_list.append(column['column_name'] + '=' + column['data_type'])

            table_columns.append(self._get_column_definition(column))

        table_configuration['column_definition']['columns'] = table_columns
        table_configuration['sqoop_options']['map-column-hive'] = ','.join(map_column_hive_fields_list)

        return table_configuration



import os.path

from injector import inject


class SSHClient(object):
    """Wrapper to use the SSH client library"""

    @inject(ssh_client='ssh_lib_client', logger='logger')
    def __init__(self, ssh_client, logger, pem_files_dir=False):
        """
        Initialize the class
        :param ssh_client: paramiko.SSHClient
        :param logger: Logging
        :param pem_files_dir: string
        """
        super(SSHClient, self).__init__()
        self.__ssh_client = ssh_client
        self.__logger = logger
        self.__pem_files_dir = os.path.join(os.path.expanduser('~'),
                                           '.ssh/') if False == pem_files_dir else pem_files_dir

    def get_pem_path(self, pem_file):
        """
        Return the full path to the pem file based on the given name and configured directory where all private keys are
        :param pem_file: string
        :return: string
        """
        return os.path.join(self.__pem_files_dir, pem_file + '.pem')

    def __connect_to_cluster(self, public_dns, pem_file):
        self.__ssh_client.connect(
            public_dns,
            username='hadoop',
            key_filename=self.get_pem_path(pem_file)
        )

    def exec_command(self, command, public_dns, pem_file, stop_on_error=False):
        """
        Execute given command in a machine via SSH
        :param command: string
        :param public_dns: string
        :param pem_file: string
        :param stop_on_error: boolean
        :return: string
        """
        self.__connect_to_cluster(public_dns, pem_file)

        self.__logger.debug(
            'Executing command {command} in machine {public_dns}'.format(command=command, public_dns=public_dns))
        stdin, stdout, stderr = self.__ssh_client.exec_command(command)

        if stop_on_error:
            error = stderr.read().strip('\n')
            if error:
                raise RuntimeError(error)

        output = ''
        for line in stdout:
            output += line.strip('\n') + ' '

        self.__ssh_client.close()

        return output

    def open_sftp(self, public_dns, pem_file):
        """
        Open an SFTP connection to the given machine
        :param public_dns: string
        :param pem_file: string
        :return: SFTP
        """
        self.__connect_to_cluster(public_dns, pem_file)
        self.__logger.debug('Opening SFTP connection to {public_dns}'.format(public_dns=public_dns))
        return self.__ssh_client.open_sftp()



from injector import inject


class ClusterId(object):
    """Get the cluster id interactively asking user in the CLI"""

    @inject(aws_emr_client='aws_emr_client')
    def __init__(self, aws_emr_client):
        """
        Initialize the class
        :param aws_emr_client: EMR.client
        """
        super(ClusterId, self).__init__()

        self.__aws_emr_client = aws_emr_client

    def get(self):
        """
        Ask the user to select one cluster from the currently active
        :return: string
        """
        print 'Cluster id has not been provided, getting the cluster currently active to select one... please be patient'
        cluster_list = self.__aws_emr_client.list_clusters(
            ClusterStates=['STARTING', 'BOOTSTRAPPING', 'RUNNING', 'WAITING'])

        if not cluster_list['Clusters']:
            raise LookupError('There are not available cluster to upload the code')

        for index, cluster in enumerate(cluster_list['Clusters']):
            print '[{index}]\tCluster Name: {name}\tCluster Id: {id}'.format(index=index, name=cluster['Name'],
                                                                             id=cluster['Id'])

        index = int(raw_input('Select cluster to deploy in: '))
        if index > len(cluster_list['Clusters']):
            raise LookupError('Selected cluster not in list')

        return cluster_list['Clusters'][index]['Id']



import ConfigParser
import logging
from argparse import Namespace

import paramiko
from boto3.session import Session
from injector import Module, provides, inject, InstanceProvider, singleton
from openpyxl import Workbook

from .cli.interactive.cluster_id import ClusterId
from .cli.interactive.configuration_file import ConfigurationFile
from .cli.interactive.default_configuration import DefaultConfiguration
from .cli.interactive.properties_file import PropertiesFile
from .cli.objects.wf_configuration_object import WfConfigurationObject
from .cli.scripts.tables_configuration.incremental_table_configuration import IncrementalTablesConfiguration
from .cli.scripts.tables_configuration.others_table_configuration import OthersTableConfiguration
from .cli.scripts.tables_configuration.tables_configuration import TablesConfiguration
from .cli.workflow_configuration import WorkflowConfiguration
from .databases.db_factory import DBFactory
from .databases.drivers.sqlserver import Sqlserver
from .databases.drivers.postgresql import Postgresql
from .emr.cluster import EmrCluster
from .emr.deploy import EmrDeploy
from .emr.job_flow.configuration import JobFlowConfigurationParser
from .emr.job_flow.run import RunJobFlow
from .filesystem.excel_writer import ExcelWriter
from .filesystem.filesystem import Filesystem
from .filesystem.hdfs import HDFSFilesystem
from .filesystem.yaml_configuration import YamlConfiguration, WorkflowsYamlConfigurationWriter
from .generator.hive_templates import HiveTemplatesRender
from .generator.templates_lib import TemplatesLib
from .generator.wf_templates import WfTemplatesRender
from .provision.client import AnsibleClient


@inject(_args=Namespace)
class DIModule(Module):
    def configure(self, binder):
        binder.bind('ansible_client', to=AnsibleClient)
        binder.bind('args', to=InstanceProvider(self._args), scope=singleton)
        binder.bind('emr_cluster', to=EmrCluster)
        binder.bind('emr_deploy', to=EmrDeploy)
        binder.bind('excel_writer', to=ExcelWriter, scope=singleton)
        binder.bind('filesystem', to=Filesystem, scope=singleton)
        binder.bind('hdfs', to=HDFSFilesystem)
        binder.bind('hive_templates_render', to=HiveTemplatesRender)
        binder.bind('interactive_cluster_id', to=ClusterId, scope=singleton)
        binder.bind('interactive_configuration_file', to=ConfigurationFile, scope=singleton)
        binder.bind('interactive_default_configuration', to=DefaultConfiguration)
        binder.bind('interactive_properties_file', to=PropertiesFile)
        binder.bind('job_flow', to=RunJobFlow)
        binder.bind('job_flow_configuration', to=JobFlowConfigurationParser)
        binder.bind('object_configuration', to=WfConfigurationObject, scope=singleton)
        binder.bind('templates_lib', to=TemplatesLib)
        binder.bind('wf_configuration', to=WorkflowConfiguration)
        binder.bind('wf_templates_render', to=WfTemplatesRender)
        binder.bind('yaml_configuration', to=YamlConfiguration)
        binder.bind('yaml_configuration_writer', to=WorkflowsYamlConfigurationWriter)

    @provides('ssh_lib_client')
    def ssh_client(self):
        ssh_client = paramiko.SSHClient()
        ssh_client.set_missing_host_key_policy(paramiko.AutoAddPolicy())

        return ssh_client

    @provides('workbook')
    def workbook(self):
        return Workbook(write_only=True)

    @provides('configuration_parser')
    def config_parser(self):
        return ConfigParser.SafeConfigParser()

    @provides('logger', scope=singleton)
    def logger(self):
        logger = logging.getLogger('slippinj')

        handler = logging.StreamHandler()
        handler.setFormatter(logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s'))
        logger.addHandler(handler)

        return logger


@inject(_profile=str)
class AwsClientModule(Module):
    @provides('aws_emr_client')
    def aws_emr_client(self):
        aws_profile = 'default' if False == self._profile else self._profile
        aws_session = Session(profile_name=aws_profile)

        return aws_session.client('emr')


class DatabaseDriversModule(Module):
    def configure(self, binder):
        binder.bind('db_factory', to=DBFactory, scope=singleton)
        binder.bind('database_driver_sqlserver', to=Sqlserver, scope=singleton)
        binder.bind('database_driver_postgresql', to=Postgresql, scope=singleton)


class TablesConfigurationModule(Module):
    def configure(self, binder):
        binder.bind('tables_configuration', to=TablesConfiguration, scope=singleton)
        binder.bind('incremental_table_configuration', to=IncrementalTablesConfiguration, scope=singleton)
        binder.bind('others_table_configuration', to=OthersTableConfiguration, scope=singleton)



class ApplicationsArgument(object):
    """Parse the applications argument"""

    def __init__(self):
        """
        Initialize the class
        """
        super(ApplicationsArgument, self).__init__()

    def parse(self, configuration):
        """
        Parse applications from the configuration file and return it formatted
        :param configuration: dict
        :return: dict
        """

        return {
            'Applications': [{'Name': v} for v in configuration]
        }



import os

from .basic_script import BasicScript


class Tlacuilo(BasicScript):
    """Compile the XML code for the workflows based on the given templates and configuration files"""

    def get_arguments(self):
        """
        Get the arguments to configure current script
        :return: list
        """
        return [
            {
                'short': '-w',
                'long': '--wf-dir',
                'help': 'Folder where all the workflows files are present',
                'required': True
            },
            {
                'short': '-t',
                'long': '--template-dir',
                'help': 'Folder where all the templates can be found, it should be the root folder inside them all the templates can be found',
                'required': True
            },
            {
                'short': '-o',
                'long': '--output-dir',
                'help': 'Directory where to store the compiled code'
            },
            {
                'short': '-e',
                'long': '--extra',
                'help': 'Extra variables to send to the templates',
                'action': 'append'
            },
            {
                'short': '-c',
                'long': '--configuration-file',
                'help': 'File where all the configuration is stored'
            },
            {
                'short': '-b',
                'long': '--hive-metastore-bucket',
                'help': 'Bucket where all data is saved',
                'default': False
            },
            {
                'short': '-f',
                'long': '--hdfs-deploy-folder',
                'help': 'Folder where all the code will be deployed on HDFS',
                'default': False
            },
            {
                'short': '-i',
                'long': '--cluster-information',
                'help': 'Try to get cluster information in order to replace variables in the template',
                'default': False
            }
        ]

    def run(self, args, injector):
        """
        Run the component to compile the XML workflows using the given templates and configuration files
        :param args: Namespace
        :param injector: Injector
        """
        logger = injector.get('logger')

        output_directory = args.output_dir if None != args.output_dir else injector.get(
            'filesystem').generate_temp_dir()

        if not os.path.isabs(output_directory):
            raise IOError('Output directory should be an absolute path in order to save the files')

        logger.info('Getting configuration from files')
        configuration = self.get_wf_configuration(args, injector)
        if args.cluster_information:
            for k, v in injector.get('emr_cluster').get_cluster_information(args.cluster_information).items():
                configuration[k] = v

        configuration.output_directory = output_directory

        if type(args.extra) == list:
            logger.info('Generating configuration from extra variables')
            for value in args.extra:
                value_splitted = value.split('=')
                if len(value_splitted) > 1:
                    configuration[value_splitted[0]] = value_splitted[1]
                else:
                    configuration[value] = ''

        logger.info('Rendering workflows code')
        injector.get('wf_templates_render').render_workflow_folder(
            args.template_dir,
            configuration.template,
            output_directory,
            configuration
        )

        if 'hive_template' in configuration:
            logger.info('Rendering hive templates')
            for table_type in ['incremental_tables', 'snapshot_tables', 'other_tables']:
                if table_type in configuration:
                    injector.get('hive_templates_render').render_hive_folder(
                        args.template_dir,
                        configuration['hive_template'],
                        os.path.join(output_directory, 'hive'),
                        configuration[table_type]
                    )



from .basic_script import BasicScript


class Cooper(BasicScript):
    """Run the workflows once has been uploaded"""

    def get_arguments(self):
        """
        Get the arguments to configure current script
        :return: list
        """
        return [
            {
                'short': '-c',
                'long': '--cluster-id',
                'help': 'Provide a cluster id to work with, in case you don\'t provide one a list of available clusters will be shown to select one',
                'default': False
            },
            {
                'short': '-w',
                'long': '--wf-dir',
                'help': 'Folder where all the workflows files are present',
                'required': True
            }
        ]

    def run(self, args, injector):
        """
        Run the component to run the workflows in the cluster
        :param args: Namespace
        :param injector: Injector
        """
        configuration = self.get_wf_configuration(args, injector)
        wf_compiled_dir = configuration.output_directory if configuration.output_directory else args.wf_dir
        cluster_id = configuration.cluster_id if configuration.cluster_id else args.cluster_id

        properties_file = injector.get('interactive_properties_file').get(wf_compiled_dir)
        if properties_file:
            injector.get('logger').info(
                'Running {properties_file} in cluster {cluster_id}'.format(properties_file=properties_file,
                                                                           cluster_id=cluster_id))
            injector.get('emr_deploy').run_properties_file(properties_file, cluster_id)



class InstancesArgument(object):
    """Parse the instances argument"""

    def __init__(self):
        """
        Initialize the class
        """
        super(InstancesArgument, self).__init__()

    def __get_instances_groups(self, configuration):
        groups = []

        for instance_type in sorted(configuration):
            if type(configuration[instance_type]) is dict:
                groups.append({
                    'Name': instance_type,
                    'Market': 'ON_DEMAND',
                    'InstanceRole': instance_type.upper(),
                    'InstanceType': configuration[instance_type]['instance_type'],
                    'InstanceCount': 1 if 'instance_count' not in configuration[instance_type] else
                    configuration[instance_type]['instance_count']
                })

        return groups

    def parse(self, configuration):
        """
        Parse instances key from the configuration file and return it formatted
        :param configuration: dict
        :return: dict
        """
        return {
            'Instances': {
                'Ec2KeyName': configuration['ec2_key_name'],
                'KeepJobFlowAliveWhenNoSteps': configuration['keep_alive'],
                'TerminationProtected': configuration['termination_protected'],
                'Ec2SubnetId': configuration['subnet_id'],
                'EmrManagedMasterSecurityGroup': configuration['master']['security_group'],
                'EmrManagedSlaveSecurityGroup': configuration['core']['secutiry_group'],
                'InstanceGroups': self.__get_instances_groups(configuration)
            }
        }



class BasicScript(object):
    def __init__(self, parser):
        """
        Initialize the class
        :param parser: ArgumentParser
        """
        super(BasicScript, self).__init__()
        self._parser = parser

    def get_arguments(self):
        """
        Get the arguments to configure current script, should be implementd in children classes
        :return: list
        """
        raise StandardError('Implement get_arguments method')

    def run(self, args, injector):
        raise StandardError('Implement run method')

    def configure(self):
        """
        Configure the component before running it
        :rtype: Class instance
        """
        self.__set_arguments()

        return self

    def __set_arguments(self):
        parser = self._parser.add_parser(self.__class__.__name__.lower(), help=self.__class__.__doc__,
                                         conflict_handler='resolve')

        arguments = self.get_arguments()

        for argument in arguments:
            short = argument['short']
            long = argument['long']
            del argument['short']
            del argument['long']
            parser.add_argument(short, long, **argument)

    def get_wf_configuration(self, args, injector):
        object_configuration = injector.get('object_configuration')

        if 1 >= len(object_configuration):
            configuration_file = args.configuration_file if 'configuration_file' in args and None != args.configuration_file else injector.get(
                'interactive_configuration_file').get(args.wf_dir)

            configuration = injector.get('wf_configuration').get_workflow_configuration(configuration_file)

            configuration = dict(
                injector.get('interactive_default_configuration').get('devel', args, configuration).items()
                + configuration.items()
            )

            for key in configuration:
                object_configuration[key] = configuration[key]

        return object_configuration



from injector import inject

import os.path


class EmrDeploy(object):
    """Execute all the deploy process inside an EMR cluster"""

    @inject(emr_cluster='emr_cluster', hdfs='hdfs', filesystem='filesystem')
    def __init__(self, emr_cluster, hdfs, filesystem):
        """
        Initialize the class
        :param emr_cluster: EmrCluster
        :param hdfs: HDFSFilesystem
        :param filesystem: Filesystem
        """
        super(EmrDeploy, self).__init__()
        self.__emr_cluster = emr_cluster
        self.__filesystem = filesystem
        self.__hdfs = hdfs

        self._base_remote_dir = '/tmp/workflows'

    def run_properties_file(self, properties_file, cluster_id):
        """
        Try to execute the given properties file in the selected cluster. It returns the job id so can check the output log in Oozie
        :param properties_file: string
        :param cluster_id: string
        :return: string
        """
        remote_properties_path = os.path.join(self._base_remote_dir, os.path.basename(properties_file))

        self.__emr_cluster.open_sftp(cluster_id).put(properties_file, remote_properties_path)

        job_id = self.__emr_cluster.exec_command('oozie job -run -config ' + remote_properties_path, cluster_id,
                                                 stop_on_error=True)

        self.__emr_cluster.exec_command('rm ' + remote_properties_path, cluster_id)

        return job_id

    def upload_code(self, wf_folder, cluster_id, hdfs_deploy_folder, workflow_name=None):
        """
        Upload given workflow code to HDFS connecting to given cluster
        :param wf_folder: string
        :param cluster_id: string
        :param hdfs_deploy_folder: string
        :param workflow_name: string
        :return: boolean
        """
        basename = wf_folder.strip(os.sep).split(os.sep)[-1] if not workflow_name else workflow_name

        tar_file = self.__filesystem.generate_tar_file(wf_folder, basename)

        remote_file = os.path.join(self._base_remote_dir, basename + '.tar.gz')

        sftp_client = self.__emr_cluster.open_sftp(cluster_id)
        self.__emr_cluster.exec_command('rm -Rf ' + self._base_remote_dir, cluster_id)

        try:
            sftp_client.mkdir(self._base_remote_dir)
            sftp_client.put(tar_file, remote_file)
        except IOError:
            return False

        self.__emr_cluster.exec_command('tar --directory ' + self._base_remote_dir + ' -zxf ' + remote_file, cluster_id)

        self.__hdfs.rmdir(hdfs_deploy_folder, cluster_id)
        self.__hdfs.mkdir(hdfs_deploy_folder, cluster_id)
        self.__hdfs.put(os.path.join(self._base_remote_dir, basename, '*'), hdfs_deploy_folder, cluster_id)

        return True



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
@author: SCMSpain
"""
from codecs import open
from os import path

from setuptools import setup, find_packages

here = path.abspath(path.dirname(__file__))
with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='slippinj',
    version='1.3.1',
    author='Data Architects SCM Spain',
    author_email='data.architecture@scmspain.com',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    include_package_data=True,
    url='https://github.com/scm-spain/slippin-jimmy',
    description='Tools to generate and deploy Apache Oozie workflows',
    long_description=long_description,
    license='GPLv2',
    install_requires=open('requirements.txt').read().split(),
    scripts=['scripts/jimmy'],
    classifiers=[
        'Topic :: Software Development :: Build Tools',
        'Topic :: Software Development :: Code Generators',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'License :: OSI Approved :: GNU General Public License v2 (GPLv2)',
        'Intended Audience :: Developers',
        'Environment :: Console'
    ],
    keywords='oozie workflows code generation emr aws'
)



import os

__all__ = []
for module in os.listdir(os.path.dirname(__file__)):
    if module == '__init__.py' or module[-3:] != '.py':
        continue
    __import__(module[:-3], locals(), globals())
    __all__.append(module[:-3])
del module


def parser_exists(argument_type):
    return argument_type in __all__


def get_parser(argument_type):
    return eval(argument_type + '.' + argument_type.capitalize() + 'Argument()')



from injector import inject, AssistedBuilder

from slippinj.cli.ssh import SSHClient


class EmrCluster(object):
    """Handle all EMR cluster information and configuration"""

    @inject(aws_emr_client='aws_emr_client', ssh_client=AssistedBuilder(SSHClient), args='args', logger='logger')
    def __init__(self, aws_emr_client, ssh_client, args, logger):
        """
        Initialize the class
        :param aws_emr_client: EMR.client
        :param ssh_client: paramiko.SSHClient
        :param args: Namespace
        :param logger: Logging
        """
        super(EmrCluster, self).__init__()
        self.__aws_emr_client = aws_emr_client
        self.__ssh_client = ssh_client.build(pem_files_dir=args.pem_dir)
        self.__cluster_information = {}
        self.__logger = logger

    def __get_cluster_environment(self, tags):
        for tag in tags:
            if 'environment' == tag['Key'].lower():
                return tag['Value']

        return False

    def __get_cluster(self, cluster_id):
        cluster = self.__aws_emr_client.describe_cluster(ClusterId=cluster_id)
        instances = self.__aws_emr_client.list_instances(ClusterId=cluster_id, InstanceGroupTypes=['MASTER'])

        return dict(cluster.items() + instances.items())

    def __get_master_ips(self, cluster_information):
        for instance in cluster_information['Instances']:
            if cluster_information['Cluster']['MasterPublicDnsName'] == instance['PublicDnsName']:
                return instance['PublicIpAddress'], instance['PrivateIpAddress']

    def get_cluster_information(self, cluster_id):
        """
        Get the cluster information from the AWS API
        :param cluster_id: string
        :return: dict
        """
        if not cluster_id in self.__cluster_information:
            self.__logger.debug('Getting information from AWS for cluster {cluster_id}'.format(cluster_id=cluster_id))
            cluster_information = self.__get_cluster(cluster_id)

            cluster_ips = self.__get_master_ips(cluster_information)

            self.__cluster_information[cluster_id] = {
                'public_dns': cluster_information['Cluster']['MasterPublicDnsName'],
                'public_ip': cluster_ips[0],
                'private_ip': cluster_ips[1],
                'environment': self.__get_cluster_environment(cluster_information['Cluster']['Tags']),
                'key_name': cluster_information['Cluster']['Ec2InstanceAttributes']['Ec2KeyName']
            }

        return self.__cluster_information[cluster_id]

    def exec_command(self, command, cluster_id, stop_on_error=False):
        """
        Execute given command in the master of the selected cluster
        :param command: string
        :param cluster_id: string
        :param stop_on_error: boolean
        :return: string
        """
        cluster_information = self.get_cluster_information(cluster_id)

        self.__logger.debug(
            'Executing command {command} in cluster {cluster_id}'.format(command=command, cluster_id=cluster_id))
        return self.__ssh_client.exec_command(command, cluster_information['public_dns'],
                                              cluster_information['key_name'], stop_on_error)

    def open_sftp(self, cluster_id):
        """
        Open an SFTP connection to the given cluster
        :param cluster_id: string
        :return: SFTP
        """
        cluster_information = self.get_cluster_information(cluster_id)

        self.__logger.debug('Opening SFTP connection to cluster {cluster_id}'.format(cluster_id=cluster_id))
        return self.__ssh_client.open_sftp(cluster_information['public_dns'], cluster_information['key_name'])

    def get_pem_path(self, cluster_id):
        """
        Get the path to the private key file to connect to the cluster
        :param cluster_id: string
        :return: string
        """
        cluster_information = self.get_cluster_information(cluster_id)

        return self.__ssh_client.get_pem_path(cluster_information['key_name'])



class StepsArgument(object):
    """Parse the steps argument"""

    def __init__(self):
        """
        Initialize the class
        """
        super(StepsArgument, self).__init__()

    def parse(self, configuration):
        """
        Parse steps from the configuration file and return it formatted
        :param configuration: dict
        :return: dict
        """
        steps = []

        for step in configuration:
            steps.append({
                'Name': step['name'],
                'ActionOnFailure': step['action_on_failure'],
                'HadoopJarStep': {
                    'Jar': step['jar'],
                    'Args': step['arguments']
                }
            })

        return {
            'Steps': steps
        }



from injector import inject


class ConfigurationFile(object):
    """Get the configuration file to use when managing workflows"""

    @inject(yaml_configuration='yaml_configuration')
    def __init__(self, yaml_configuration):
        """
        Initialize the class
        :param yaml_configuration: YamlConfiguration
        """
        super(ConfigurationFile, self).__init__()

        self.__yaml_configuration = yaml_configuration

    def get(self, wf_dir):
        """
        Ask user to select which configuration files would like to use from all the files found
        :param wf_dir: string
        :return: list
        """
        config_files_found = self.__yaml_configuration.autodiscover_config_files(wf_dir)
        if 0 == len(config_files_found):
            raise IOError('No configuration files has been found')
        elif 1 == len(config_files_found):
            config_files = config_files_found
        else:
            print 'Select the configuration files you want to process'
            for index, cf in enumerate(config_files_found):
                print '[{index}] {config_file}'.format(index=index, config_file=cf.replace(wf_dir, ''))

            selected_files = raw_input('For multiple files separate them using ",": ')
            if 1 > len(selected_files):
                raise ValueError('You must select at least one configuration file to process the workflows')

            config_files = []
            for index in selected_files.split(','):
                try:
                    config_files.append(config_files_found[int(index)])
                except ValueError:
                    pass

        return config_files



import glob
import os

import yaml
from injector import inject


class YamlConfiguration(object):
    """Process YAML configuration files"""

    @inject(logger='logger')
    def __init__(self, logger):
        """
        Initialize the class
        :param logger: Logging
        """
        super(YamlConfiguration, self).__init__()
        yaml.add_constructor('!include', self.__yaml_include)
        self.__logger = logger

    def __yaml_include(self, loader, node):
        file_name = os.path.join(os.path.dirname(loader.name), node.value)

        with file(file_name) as inputfile:
            return yaml.load(inputfile)

    def read_config_file(self, config_file):
        """
        Read the given configuration file
        :param config_file: string
        :return: dict
        """
        if not os.path.exists(config_file):
            raise IOError('Configuration file ' + config_file + ' not found.')

        return yaml.load(open(config_file, 'r'))

    def read_multiple_config_files(self, configuration_files):
        """
        Given multiple YAML files read all and merge there content in the given order, overriding same configuration keys
        :param configuration_files: list
        :return: dict
        """
        configuration = {}

        for config_file in configuration_files:
            configuration = dict(configuration.items() + self.read_config_file(config_file).items())

        return configuration

    def autodiscover_config_files(self, root_dir):
        """
        Find all the YAML files inside the given directory
        :param root_dir: string
        :return: list
        """
        return glob.glob(os.path.join(root_dir, '*.yml')) + glob.glob(os.path.join(root_dir, '**', '*.yml'))


class WorkflowsYamlConfigurationWriter(object):
    """Write the YAML configuration files needed to run the workflows"""

    @inject(filesystem='filesystem', tables_configuration='tables_configuration', logger='logger')
    def __init__(self, filesystem, tables_configuration, logger):
        """
        Initialize the class
        :param filesystem: Filesystem
        :param tables_configuration: TablesConfiguration
        :param logger: Logging
        """
        super(WorkflowsYamlConfigurationWriter, self).__init__()
        self.__filesystem = filesystem
        self.__tables_configuration = tables_configuration
        self.__logger = logger

    def generate_yaml_files(self, injector, tables_information, db_driver, db_host, db_port, db_user, db_name,
                            db_password, location=None):
        """
        Given tables and database connection information generate the YAML files and save them into provided location
        :param injector: Injector
        :param tables_information: dict
        :param db_driver: string
        :param db_host: string
        :param db_port: integer
        :param db_user: string
        :param db_name: string
        :param db_password: string
        :param location: string
        """
        common_data = {
            'table_base_location': 's3://${hiveMetastoreBucket}/',
            'db_connection_string': 'jdbc:' + db_driver + '://' + db_host + ((':' + db_port) if db_port else ''),
            'db_username': db_user,
            'hive_db_name': db_name,
            'hive_tmp_db_name': 'tmp_' + db_name,
            'hdfs_deploy_folder': ''
        }

        if db_password:
            common_data['db_password'] = db_password
            common_data['db_passphrase'] = ''
            common_data['db_salt'] = ''

        tables_output = yaml.safe_dump(self.__tables_configuration.generate_configuration(tables_information, injector),
                                       default_flow_style=False, explicit_start=False, encoding='utf-8',
                                       allow_unicode=True)
        common_output = yaml.safe_dump(common_data, default_flow_style=False, explicit_start=False, encoding='utf-8',
                                       allow_unicode=True)

        location = os.getcwd() if None == location else location

        self.__logger.debug('Saving tables configuration YAML file into {location}'.format(location=location))
        self.__filesystem.write_file(os.path.join(location, 'tables.yml'), tables_output)
        self.__logger.debug('Saving common configuration YAML file into {location}'.format(location=location))
        self.__filesystem.write_file(os.path.join(location, 'common.yml'), common_output)



from injector import Error, AssistedBuilder


class DBFactory(object):
    """Return instances of the databases"""

    def get_driver(self, injector, driver, db_host='localhost', db_user='root', db_pwd=None, db_port=None,
                   db_name=None, db_schema=None):
        """
        Return a connection to the given database using selected driver. In case given driver is not implemented a NotImplementedError is raised
        :param injector: Injector
        :param driver: string
        :param db_host: string
        :param db_user: string
        :param db_pwd: string
        :param db_port: int
        :param db_name: string
        :param db_schema: string
        :return: Selected driver instance
        """
        try:
            return injector.get(AssistedBuilder('database_driver_' + driver.lower())).build(db_host=db_host,
                                                                                            db_user=db_user,
                                                                                            db_pwd=db_pwd,
                                                                                            db_port=db_port,
                                                                                            db_name=db_name,
                                                                                            db_schema=db_schema)
        except Error:
            raise NotImplementedError('Given driver has not been implemented on SlippinJ')
